// Prevents additional console window on Windows in release, DO NOT REMOVE!!
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]
use id3::{Tag, TagLike};
use serde::{Deserialize, Serialize};
use std::collections::HashSet;
use tauri::Manager;
use walkdir::WalkDir;

#[derive(Debug, Clone, Serialize, PartialEq, Eq, Hash)]
pub struct LocalAlbum {
    pub name: String,
    pub artist: String,
    pub year: i32,
    pub lossless: bool,
}

#[tauri::command]
async fn get_local_albums(
    dirpath: String,
    app_handle: tauri::AppHandle,
) -> Result<Vec<LocalAlbum>, String> {
    let mut result = HashSet::new();
    for entry in WalkDir::new(dirpath).into_iter().filter_map(|e| e.ok()) {
        if entry.file_type().is_file() {
            let path = entry.path();
            if let Ok(tag) = Tag::read_from_path(path) {
                let album = LocalAlbum {
                    name: tag.album().unwrap_or("").to_string(),
                    artist: tag.artist().unwrap_or("").to_string(),
                    year: tag.year().unwrap_or(0),
                    lossless: entry
                        .path()
                        .extension()
                        .map(|ext| ext == "flac" || ext == "wav")
                        .unwrap_or(false),
                };
                let json_local_album = serde_json::to_value(&album).unwrap();
                let new = result.insert(album);
                if new {
                    app_handle
                        .emit_all("new_local_album", json_local_album)
                        .unwrap();
                }
            }
        }
    }
    Ok(result.into_iter().collect())
}

#[derive(Debug, Serialize, PartialEq, Eq, Hash)]
pub struct ScrobbledAlbum {
    pub name: String,
    pub artist: String,
    pub playcount: u64,
}

#[derive(Debug, Deserialize)]
pub enum Period {
    Overall,
    Week,
    Month,
    ThreeMonths,
    SixMonths,
    Year,
}

impl ToString for Period {
    fn to_string(&self) -> String {
        match self {
            Period::Overall => "overall",
            Period::Week => "7day",
            Period::Month => "1month",
            Period::ThreeMonths => "3month",
            Period::SixMonths => "6month",
            Period::Year => "12month",
        }
        .to_string()
    }
}

#[tauri::command]
async fn get_top_scrobbled_albums(
    user: String,
    period: Period,
    limit: u64,
    api_key: Option<String>,
    app_handle: tauri::AppHandle,
) -> Result<Vec<ScrobbledAlbum>, String> {
    let api_key = api_key
        .or_else(|| option_env!("LASTFM_API_KEY").map(|s| s.to_string()))
        .ok_or("No API key supplied.")?;

    let period = period.to_string();

    let page_size = 50;
    let mut final_result = Vec::new();
    let mut page = 1;
    while final_result.len() < limit as usize {
        let url = format!(
            "https://ws.audioscrobbler.com/2.0/?method=user.gettopalbums&user={user}&api_key={api_key}&period={period}&limit={page_size}&page={page}&format=json");
        println!("Fetching {}", url);

        let resp = reqwest::get(&url)
            .await
            .map_err(|e| e.to_string())?
            .json::<serde_json::Value>()
            .await
            .map_err(|e| e.to_string())?;

        let albums = resp["topalbums"]["album"]
            .as_array()
            .ok_or("no albums found")?;

        let mut result = Vec::new();
        for album in albums {
            let name = album["name"].as_str().ok_or("no album name")?.to_string();
            let artist = album["artist"]["name"]
                .as_str()
                .ok_or("no artist name")?
                .to_string();
            let playcount = album["playcount"]
                .as_str()
                .ok_or("no playcount")?
                .parse()
                .map_err(|_| "failed to parse playcount")?;
            result.push(ScrobbledAlbum {
                name,
                artist,
                playcount,
            });
        }
        app_handle
            .emit_all(
                "new_scrobbled_albums",
                serde_json::to_value(&result).unwrap(),
            )
            .unwrap();
        final_result.extend(result);

        page += 1;
    }

    Ok(final_result)
}

fn main() {
    // https://github.com/tauri-apps/tauri/issues/5143#issuecomment-1311815517
    std::env::set_var("WEBKIT_DISABLE_COMPOSITING_MODE", "1");

    tauri::Builder::default()
        .invoke_handler(tauri::generate_handler![
            get_top_scrobbled_albums,
            get_local_albums
        ])
        .run(tauri::generate_context!())
        .expect("error while running tauri application");
}
